@@ -30,15 +30,14 @@ use std::net::SocketAddr;
 use std::time::Duration;
 
 use crate::types::error::CallError;
-use crate::types::{self, ErrorResponse, Response, SubscriptionId};
+use crate::types::{Response, SubscriptionId};
 use crate::{future::ServerHandle, RpcModule, WsServerBuilder};
 use anyhow::anyhow;
 use futures_util::future::join;
-use jsonrpsee_core::{to_json_raw_value, traits::IdProvider, DeserializeOwned, Error};
+use jsonrpsee_core::{traits::IdProvider, DeserializeOwned, Error};
 use jsonrpsee_test_utils::helpers::*;
 use jsonrpsee_test_utils::mocks::{Id, TestContext, WebSocketTestClient, WebSocketTestError};
 use jsonrpsee_test_utils::TimeoutFutureExt;
-use jsonrpsee_types::error::invalid_subscription_err;
 use serde_json::Value as JsonValue;
 use tracing_subscriber::{EnvFilter, FmtSubscriber};
 
@@ -184,7 +183,7 @@ async fn can_set_the_max_request_body_size() {
 	init_logger();
 
 	let addr = "127.0.0.1:0";
-	// Rejects all requests larger than 10 bytes
+	// Rejects all requests larger than 100 bytes
 	let server = WsServerBuilder::default().max_request_body_size(100).build(addr).await.unwrap();
 	let mut module = RpcModule::new(());
 	module.register_method("anything", |_p, _cx| Ok("a".repeat(100))).unwrap();
@@ -198,6 +197,28 @@ async fn can_set_the_max_request_body_size() {
 	let response = client.send_request_text(req).await.unwrap();
 	assert_eq!(response, oversized_request());
 
+	// Max request body size should not override the max response body size
+	let req = r#"{"jsonrpc":"2.0", "method":"anything", "id":1}"#;
+	let response = client.send_request_text(req).await.unwrap();
+	assert_eq!(response, ok_response(JsonValue::String("a".repeat(100)), Id::Num(1)));
+
+	handle.stop().unwrap();
+}
+
+#[tokio::test]
+async fn can_set_the_max_response_body_size() {
+	init_logger();
+
+	let addr = "127.0.0.1:0";
+	// Set the max response body size to 100 bytes
+	let server = WsServerBuilder::default().max_response_body_size(100).build(addr).await.unwrap();
+	let mut module = RpcModule::new(());
+	module.register_method("anything", |_p, _cx| Ok("a".repeat(101))).unwrap();
+	let addr = server.local_addr().unwrap();
+	let handle = server.start(module).unwrap();
+
+	let mut client = WebSocketTestClient::new(addr).await.unwrap();
+
 	// Oversized response.
 	let req = r#"{"jsonrpc":"2.0", "method":"anything", "id":1}"#;
 	let response = client.send_request_text(req).await.unwrap();
@@ -606,16 +627,13 @@ async fn unsubscribe_twice_should_indicate_error() {
 	let sub_id: u64 = deser_call(client.send_request_text(sub_call).await.unwrap());
 
 	let unsub_call = call("unsubscribe_hello", vec![sub_id], Id::Num(1));
-	let unsub_1: String = deser_call(client.send_request_text(unsub_call).await.unwrap());
-	assert_eq!(&unsub_1, "Unsubscribed");
+	let unsub_1: bool = deser_call(client.send_request_text(unsub_call).await.unwrap());
+	assert!(unsub_1);
 
 	let unsub_call = call("unsubscribe_hello", vec![sub_id], Id::Num(2));
-	let unsub_2 = client.send_request_text(unsub_call).await.unwrap();
-	let unsub_2_err: ErrorResponse = serde_json::from_str(&unsub_2).unwrap();
-	let sub_id = to_json_raw_value(&sub_id).unwrap();
+	let unsub_2: bool = deser_call(client.send_request_text(unsub_call).await.unwrap());
 
-	let err = Some(to_json_raw_value(&format!("Invalid subscription ID={}", sub_id)).unwrap());
-	assert_eq!(unsub_2_err, ErrorResponse::new(invalid_subscription_err(err.as_deref()), types::Id::Number(2)));
+	assert!(!unsub_2);
 }
 
 #[tokio::test]
@@ -624,10 +642,9 @@ async fn unsubscribe_wrong_sub_id_type() {
 	let addr = server().await;
 	let mut client = WebSocketTestClient::new(addr).with_default_timeout().await.unwrap().unwrap();
 
-	let unsub = client.send_request_text(call("unsubscribe_hello", vec![13.99_f64], Id::Num(0))).await.unwrap();
-	let unsub_2_err: ErrorResponse = serde_json::from_str(&unsub).unwrap();
-	let err = Some(to_json_raw_value(&"Invalid subscription ID type, must be Integer or String").unwrap());
-	assert_eq!(unsub_2_err, ErrorResponse::new(invalid_subscription_err(err.as_deref()), types::Id::Number(0)));
+	let unsub: bool =
+		deser_call(client.send_request_text(call("unsubscribe_hello", vec![13.99_f64], Id::Num(0))).await.unwrap());
+	assert!(!unsub);
 }
 
 #[tokio::test]
@@ -667,5 +684,5 @@ async fn custom_subscription_id_works() {
 	let sub = client.send_request_text(call("subscribe_hello", Vec::<()>::new(), Id::Num(0))).await.unwrap();
 	assert_eq!(&sub, r#"{"jsonrpc":"2.0","result":"0xdeadbeef","id":0}"#);
 	let unsub = client.send_request_text(call("unsubscribe_hello", vec!["0xdeadbeef"], Id::Num(1))).await.unwrap();
-	assert_eq!(&unsub, r#"{"jsonrpc":"2.0","result":"Unsubscribed","id":1}"#);
+	assert_eq!(&unsub, r#"{"jsonrpc":"2.0","result":true,"id":1}"#);
 }
